package com.example.redisCache;

import com.example.redisCache.entity.Product;
import com.example.redisCache.repository.ProductDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@SpringBootApplication
@RestController
@RequestMapping("/products")
@EnableCaching
public class SpringBootRedisCacheApplication {

	@Autowired
	private ProductDao productDao;

	@PostMapping
	//@Cacheable(value = "Product")
	public Product save(@RequestBody Product product){
		System.out.println(" >> productController : post mapping call : "+product.toString());
		return productDao.save(product);
	}

	@GetMapping
	public List<Product> getAllProducts(){
		System.out.println(" >> productController : Get mapping All call : ");
		return productDao.findAll();
	}

	@GetMapping("/{id}")
	@Cacheable(key = "#id",value = "Product") // ,unless = "#result.price < 1000"
	public Product getProductById(@PathVariable int id){
		System.out.println(" >> productController : Get mapping one call : "+id);
		return productDao.findProductById(id);
	}

	@PutMapping("/update")
	@CachePut(key = "#product.id",value = "Product")
	public Product updateProductData(@RequestBody Product product){
		System.out.println(" >> productController : Put mapping call : "+product.toString());
		return productDao.updateProduct(product);
	}

	@DeleteMapping("/{id}")
	@CacheEvict(key = "#id",value = "Product",allEntries = false)
	public String deleteById(@PathVariable int id){
		System.out.println(" >> productController : Delete mapping call : "+id);
		return productDao.deleteProductById(id);
	}

	public static void main(String[] args) {
		SpringApplication.run(SpringBootRedisCacheApplication.class, args);
	}

}
